#include "misc.h"
#include "includes.h"

pPic_t Load_Bitmap_to_Memory (char Name[]);

extern pPic_t Displayed_Picture;
extern EmbeddedFileSystem efs;
extern EmbeddedFile file_r, file_w;
extern pPic_t *Picture;
extern int PicIndex, PicCount;
extern DirList MyList;

void Display_New_Picture (int New, Point X, Point Y, int Mask, int ClearFlag)
{
  // Display the picture on the display
  PicIndex = (PicIndex + New + PicCount) % PicCount;
  GLCD_ShowPic_in_Box (Picture[PicIndex], X, Y, Mask, ClearFlag);
  return;
}

void Load_All_Bitmap_to_Memory (void)
{
  MyFile FileInfo;
  char Success;
  Success = 0;
  do
  {
    // if the current folder contains more files/folders --> get the next one
    if (ls_getNext(&MyList)==0)
    {
      // forming the name of the file/folder
      // the structure of the FileInfo consist of:
      // Name (up to 8 characters), Extension (none for folders) and 
      // FullName (both of them combined and separated by '.')
      Form_File_Name (&FileInfo, MyList.currentEntry.FileName);

      // Filtering files. Visualize olny the *.bmp files
      // if any other type --> skip to the next file (if exist)
      if (My_StrCmp (FileInfo.Extension, "bmp", NO_CASE_SENS))
        continue;

      Picture[PicIndex++] = Load_Bitmap_to_Memory (FileInfo.FullName);
    }
    else  // if the current folder DOES NOT contain more files/folders --> reinit
    {
      if (ls_openDir (&MyList, &efs.myFs, GlobalPath))
        Error();
      Success = 1;
    }
  }
  while (!Success); // the purpose of this loop is to skip the non-bitmap files
  PicCount = PicIndex;
  PicIndex = 0;
  return;
}

pPic_t Load_Bitmap_to_Memory (char Name[])
{
  pPic_t pPic;
  unsigned long int size;
  unsigned char *bmp;
  unsigned char buf[54];
  char FullPathName[MAX_PATH_LENGTH + MAX_FILE_NAME_LENGTH + MAX_FILE_EXT_LENGTH];
  
  // check the current directory
  if (ls_openDir (&MyList, &efs.myFs, GlobalPath))
  {
    Write ("Error while opening the folder\n\r");
    return NULL;
  }
  
  // put  the full path and name of the file in one string
  sprintf (FullPathName, "%s%s", GlobalPath, Name);
  
  // Open the file
  if (file_fopen (&file_r, &efs.myFs, FullPathName, 'r') != 0)
  {
    Write ("Wrong file name!\n\r");
    return NULL;
  }

  // Read the header of the bitmap (first 54 bytes)
  file_read (&file_r, 54, buf);

  // Using the info in the header to calculate size of the bitmap
  size = (buf[2]<<0) + (buf[3]<<8) + (buf[4]<<16) + (buf[5]<<24);
  
  // Reset the pointer of the file
  if (file_fclose (&file_r))
    Error ();
  if (file_fopen (&file_r, &efs.myFs, FullPathName, 'r'))
    Error ();
      
  
  // Allocate enough heap-memory for the byte-array buffer to store the bitmap data
  bmp = (unsigned char*) malloc(size*sizeof(unsigned char));
  if (bmp == NULL)
    return NULL;

  // Load all the data (including the header) into a byte-array buffer
  file_read (&file_r, size, bmp);

  // Close the file
  file_fclose (&file_r);

  // Converts the byte-array into pixel-array (picture)
  pPic = GLCD_LoadPic((bmp_t *)bmp);

  // Free the allocated memory for the byte-array buffer
  free(bmp);
  
  return pPic;
}

void PrintGlobalPath (void)
{
  NewLine ();
  Write ("<root:");
  Write (GlobalPath);
  Write ("> ");
}

int Button_is_Pressed (void)
{
  return (!(BUT_FIO & BUT_MASK));
}

void LED_State (int State)
{
  if (State)
    USB_OTG_LINK_LED_FCLR = USB_OTG_LINK_LED_MASK;
  else
    USB_OTG_LINK_LED_FSET = USB_OTG_LINK_LED_MASK;
}